@@ -26,61 +26,10 @@
 # policies, either expressed or implied, of James Yates Farrimond.
 
 '''
-Recipes define subsystems that are distinct parts of larger systems.  They are
-the basic units of setup in frycook.  Generally a recipe corresponds to a
-package that needs to be installed or configured.
-
-idempotence
-===========
-
-One thing to keep in mind when creating recipes and cookbooks is idempotency.
-By keeping idempotency in mind in general you can create recipes that you can
-run again and again to push out minor changes to a package.  This way your
-recipes become the only way that you modify your servers and can be a single
-chokepoint that you can monitor to make sure things happen properly.
-
-Lots of the cuisine functions you'll use have an "ensure" version that first
-checks to see if a condition is true before applying it, such as checking if a
-package is installed before trying to install it.  This is nice when things
-could cause undesired configuration changes or expensive operations that you
-don't want to happen every time.  These functions are a huge aid in writing
-idempotent recipes and cookbooks.
-
-rudeness
-========
-
-Another thing to keep in mind is that some actions performed in recipes can
-affect the end users of the systems, in effect being rude to them.  This might
-cause an outage or otherwise mess them up.  The recipe class keeps track of
-whether or not this is ok in its 'ok_to_be_rude' variable so you can know what
-actions are acceptable.  Consult this before doing rude things.
-
-file set copying
-================
-
-The Recipe class defines a few helper functions for handling templates and
-copying files to servers.  It runs files with a .tmplt extension through Mako,
-using the dictionary you pass to it.  Regular files just get copied.  You can
-specify owner, group, and permissions on a per-directory and per-file basis.
-
-git repo checkouts
-==================
-
-The Recipe class also defines some helper functions for working with git repos.
-You can checkout a git repo onto the remote machine, or check it out locally
-and copy it to the remote machine if you don't want to setup the remote machine
-to be able to do checkouts.
-
-apply process
-=============
-
-This is where you apply a recipe to a server.  There are two class methods that
-get called during the apply process, and possibly two messages that get
-displayed.  Generally you'll just override the apply method and sometimes add
-pre_apply or post_apply messages.  If you override pre_apply_checks, remember
-to call the base class method.  Here's the order that things happen in:
-
-pre_apply_message -> pre_apply_checks() -> apply() -> post_apply_message
+Recipes define subsystems that are distinct parts of larger systems.
+They are the basic units of setup in frycook.  Generally a recipe
+corresponds to an os-level package that needs to be installed or
+configured.
 '''
 import os
 import os.path
@@ -103,13 +52,14 @@ class RecipeException(Exception):
 
 class FileMetaDataTracker(object):
     '''
-    A FileMetaDataTracker object keeps track of owner, group, and permissions
-    for files and directories during a push_package_fileset operation.  This
-    hinges on a file named fck_metadata.txt being encountered in the directory
-    being examined.  This file should have each line contain the text
-    <filename>:<owner>:<group>:<perms>, where <filename> is either a file name
-    or '.' for the directory itself, <owner> is the owner's account name,
-    <group> is the group's name, and <perms> is the permissions string..
+    A FileMetaDataTracker object keeps track of owner, group, and
+    permissions for files and directories during a push_package_fileset
+    operation.  This hinges on a file named fck_metadata.txt being
+    encountered in the directory being examined.  This file should have
+    each line contain the text <filename>:<owner>:<group>:<perms>, where
+    <filename> is either a file name or '.' for the directory itself,
+    <owner> is the owner's account name, <group> is the group's name,
+    and <perms> is the permissions string..
     '''
 
     tagfile = 'fck_metadata.txt'
@@ -122,24 +72,19 @@ class FileMetaDataTracker(object):
 
     def check_directory(self, root, dirs, files):
         '''
-        Read in the metadata for the given directory from the fck_metadata.txt
-        file in the directory, or remember metadata from previous calls if no
-        fck_metadata.txt file is encountered.
+        Read in the metadata for the given directory from the
+        fck_metadata.txt file in the directory, or remember metadata
+        from previous calls if no fck_metadata.txt file is encountered.
 
-        The input parameters are expected to be the values returned from a call
-        to os.walk()
+        The input parameters are expected to be the values returned from
+        a call to os.walk()
 
-        @type root: string
-
-        @param root: directory being examined
-
-        @type dirs: list of strings
-
-        @param dirs: list of sub-directories under the root directory
-
-        @type files: list of strings
-
-        @param files: list of the files in the root directory
+        :type root: string
+        :param root: directory being examined
+        :type dirs: list of strings
+        :param dirs: list of sub-directories under the root directory
+        :type files: list of strings
+        :param files: list of the files in the root directory
         '''
         if root not in self.metadata:
             self.metadata[root] = (None, None, None, )
@@ -167,22 +112,18 @@ class FileMetaDataTracker(object):
 
     def get_metadata(self, path, filename=''):
         '''
-        Get the owner, group, and permissions for the given path and filename.
-        If no filename is specified, the data is retrieved for the directory
+        Lookup the owner, group, and permissions for the given path and
+        filename from the metadata dictionary in this object.  If no
+        filename is specified, the data is retrieved for the directory
         specified in path.
 
-        @type path: string
+        :type path: string
+        :param path: path to file to get metadata for
+        :type filename: string
+        :param filename: name of file to get metadata for (leave blank if just getting directory metadata)
 
-        @param path: path to file to get metadata for
-
-        @type filename: string
-
-        @param filename: name of file to get metadata for (leave blank if just
-        getting directory metadata)
-
-        @rtype: tuple of strings
-
-        @return: tuple containing (<owner>, <group>, <perms>, )
+        :rtype: tuple of strings
+        :return: tuple containing (<owner>, <group>, <perms>, )
         '''
         fq = os.path.join(path, filename)
         if fq in self.metadata:
@@ -195,33 +136,26 @@ class FileMetaDataTracker(object):
 
 class FileDeleter(object):
     '''
-    A FileDeleter object deletes unwanted files from a directory.  This hinges
-    on a file named fck_delete.txt being encountered in the directory being
-    examined.  This file should have each line contain the name of a file to be
-    deleted.
+    A FileDeleter object deletes unwanted files from a directory on a
+    remote server.  This hinges on a file named fck_delete.txt being
+    encountered in the directory being examined.  This file should have
+    each line contain the name of a file to be deleted.
     '''
 
     tagfile = 'fck_delete.txt'
 
     def check_directory(self, root, files, remote_rootpath):
         '''
-        Examine the given directory, check for a fck_delete.txt file in the
-        directory, and delete all files named in fck_delete.txt.
+        Examine the given directory, check for a fck_delete.txt file in
+        the directory, and if it exists delete all remote files named in
+        it.
 
-        The first of the input parameters are expected to be values returned
-        from a call to os.walk()
-
-        @type root: string
-
-        @param root: directory being examined
-
-        @type files: list of strings
-
-        @param files: list of the files in the root directory
-
-        @type remote_rootpath: string
-
-        @param remote_rootpath: path on remote server to delete files from
+        :type root: string
+        :param root: local directory possibly containing fck_delete.txt
+        :type files: list of strings
+        :param files: list of the files in the local root directory
+        :type remote_rootpath: string
+        :param remote_rootpath: path on remote server to delete files from
         '''
 
         if self.tagfile in files:
@@ -232,46 +166,34 @@ class FileDeleter(object):
 
 class Recipe(object):
     '''
-    The Recipe class is the base class for all recipes to subclass.  It defines
-    the framework for recipes and some helper functions.  By itself it doesn't
-    do much.
+    The Recipe class is the base class for all recipes to subclass.  It
+    defines the framework for recipes and some helper functions.  By
+    itself it doesn't do much.
 
-    Here is an example that implements a basic recipe to make sure a package is
-    installed::
+    It has a set of helper functions that are hooks called by frycooker
+    to apply the recipe to a remote server::
 
-      import cuisine
+        handle_pre_apply_message()
+        handle_post_apply_message()
+        run_apply()
+        run_messages()
 
-      from frycook import Recipe
+    It has another set of helper functions used within recipes for
+    copying files to remote servers::
 
-      class RecipeFail2ban(Recipe):
-          def apply(self, computer):
-              cuisine.package_ensure('fail2ban')
+        get_local_file_perms()
+        push_file()
+        push_template()
+        push_package_file_set()
 
-    All recipe files should live in a single recipes package.  The __init__.py
-    file for this package should import all recipes and have a list of them so
-    they can easily be imported by the frycooker program.
+    It has a final set of helper functions used within recipes for
+    managing git repos on remote servers::
 
-    Example::
-
-      from fail2ban import RecipeFail2ban
-      from hosts import RecipeHosts
-      from nginx import RecipeNginx
-      from postfix import RecipePostfix
-      from root_user import RecipeRootUser
-      from example_com import RecipeExampleCom
-      from shorewall import RecipeShorewall
-      from ssh import RecipeSSH
-
-      recipes = {
-          'fail2ban': RecipeFail2ban,
-          'hosts': RecipeHosts,
-          'nginx': RecipeNginx,
-          'postfix': RecipePostfix,
-          'root_user': RecipeRootUser,
-          'example_com': RecipeExampleCom,
-          'shorewall': RecipeShorewall,
-          'ssh': RecipeSSH
-          }
+        push_git_repo()
+        clone_git_repo()
+        update_git_repo()
+        is_git_repo()
+        ensure_git_repo()
     '''
 
     def __init__(self, settings, environment, ok_to_be_rude, no_prompt):
@@ -279,21 +201,10 @@ class Recipe(object):
         Initialize the recipe object with the settings and environment
         dictionaries.
 
-        @type settings: dict
-
-        @param settings: settings dictionary
-
-        @type environment: dict
-
-        @param environment: metadata dictionary
-
-        @type ok_to_be_rude: boolean
-
-        @param ok_to_be_rude: is it ok to interrupt your users?
-
-        @type no_prompt: boolean
-
-        @param no_prompt: should we prompt the user?
+        :param dict sttings: settings dictionary
+        :param dict environment: metadata dictionary
+        :param boolean ok_to_be_rude: is it ok to interrupt your users?
+        :param boolean no_prompt: should we prompt the user?
         '''
         self.settings = settings
         self.environment = environment
@@ -350,12 +261,8 @@ class Recipe(object):
         sure to call the base class function to make sure its checks get done
         as well.
 
-        @type computer: string
-
-        @param computer: computer to apply recipe checks to
-
-        @raise RecipeException: raised if the computer name is not in the
-        environment being processed
+        :param string computer: name of computer to apply recipe checks to
+        :raises RecipeException: raised if the computer name is not in the environment being processed
         '''
         # make sure the computer is in our enviro
         if computer not in self.environment["computers"]:
@@ -369,9 +276,8 @@ class Recipe(object):
         have any actions to apply.  If you don't have any actions, then why are
         you creating a recipe?
 
-        @type computer: string
-
-        @param computer: computer to apply recipe to
+        :type computer: string
+        :param computer: name of computer to apply recipe to
         '''
         pass
 
@@ -382,11 +288,10 @@ class Recipe(object):
 
         Sequence::
 
-          pre_apply_message -> pre_apply_checks() -> apply() -> post_apply_message
+          pre_apply_checks() -> apply()
 
-        @type computer: string
-
-        @param computer: computer to apply recipe to
+        :type computer: string
+        :param computer: name of computer to apply recipe to
         '''
         self.pre_apply_checks(computer)
         self.apply(computer)
@@ -407,13 +312,11 @@ class Recipe(object):
         '''
         Get a string of the permissions set on a local file.
 
-        @type local_name: string
+        :type local_name: string
+        :param local_name: path to file on local file system
 
-        @param local_name: path to file on local file system
-
-        @rtype: string
-
-        @return: string containing perms of file, ie. '655'
+        :rtype: string
+        :return: string containing perms of file, ie. '655'
         '''
         bit_mode = stat.S_IMODE(os.stat(local_name).st_mode)
         user_perms = (bit_mode & stat.S_IRWXU) >> 6
@@ -427,26 +330,16 @@ class Recipe(object):
         Copy a file to a remote server if the file is different or doesn't
         exist.
 
-        @type local_name: string
-
-        @param local_name: path within packages dir of file to upload (path +
-        filename)
-
-        @type remote_name: string
-
-        @param remote_name: remote path to write file to (path + filename)
-
-        @type owner: string
-
-        @param owner: owner of the file
-
-        @type group: string
-
-        @param group: group of the file
-
-        @type perms: string
-
-        @param perms: permissions for the file
+        :type local_name: string
+        :param local_name: path within packages dir of file to upload (path + filename)
+        :type remote_name: string
+        :param remote_name: remote path to write file to (path + filename)
+        :type owner: string
+        :param owner: owner of the file
+        :type group: string
+        :param group: group of the file
+        :type perms: string
+        :param perms: permissions for the file, ie. '655'
         '''
         local_name = os.path.join(self.settings["package_dir"], local_name)
         cuisine.file_upload(remote_name, local_name)
@@ -461,31 +354,18 @@ class Recipe(object):
         Process a template file and push its contents to a remote server if
         it's different than what's already there.
 
-        @type templatename: string
-
-        @param templatename: path within packages dir of template file to
-        process (path + filename)
-
-        @type out_path: string
-
-        @param out_path: path on remote server to write file to (path +
-        filename)
-
-        @type enviro: dict
-
-        @param enviro: environment dictionary for template engine
-
-        @type owner: string
-
-        @param owner: owner of the templated file
-
-        @type group: string
-
-        @param group: group of the templated file
-
-        @type perms: string
-
-        @param perms: permissions for the templated file
+        :type templatename: string
+        :param templatename: path within packages dir of template file to process (path + filename)
+        :type out_path: string
+        :param out_path: path on remote server to write file to (path + filename)
+        :type enviro: dict
+        :param enviro: environment dictionary for template engine
+        :type owner: string
+        :param owner: owner of the templated file
+        :type group: string
+        :param group: group of the templated file
+        :type perms: string
+        :param perms: permissions for the templated file, ie. '655'
         '''
         mytemplate = self.mylookup.get_template(templatename)
         try:
@@ -506,14 +386,10 @@ class Recipe(object):
         push_package_file_set operation.  The calling function sets up the
         template environment, then calls this one.
 
-        @type package_name: string
-
-        @param package_name: name of package to process, corresponds to
-        directory in packages directory
-
-        @type template_env: dict
-
-        @param template_env: environment dictionary for template engine
+        :type package_name: string
+        :param package_name: name of package to process, corresponds to directory in packages directory
+        :type template_env: dict
+        :param template_env: environment dictionary for template engine
         '''
         metadata = FileMetaDataTracker()
         deleter = FileDeleter()
@@ -577,18 +453,13 @@ class Recipe(object):
         one per line.  This way you can clean out a directory as well as copy
         files to it.
 
-        @type package_name: string
+        :type package_name: string
 
-        @param package_name: name of package to process, corresponds to
-        directory in packages directory
-
-        @type template_env: dict
-
-        @param template_env: environment dictionary for template engine
-
-        @type aux_env: dict
-
-        @param aux_env: additional key/value pairs for the template environment
+        :param package_name: name of package to process, corresponds to directory in packages directory
+        :type template_env: dict
+        :param template_env: environment dictionary for template engine
+        :type aux_env: dict
+        :param aux_env: additional key/value pairs for the template environment
         '''
         template_env = {"computer":
                         self.environment["computers"][computer_name]}
@@ -605,25 +476,16 @@ class Recipe(object):
         Make a local clone of the repo in git_url into the temp directory
         specified in the settings file, then rsync it to the remote path.
 
-        @type computer: string
-
-        @param computer: computer name to push to
-
-        @type user: string
-
-        @param user: user to own the files in the repo
-
-        @type group: string
-
-        @param group: group to own the files in the repo
-
-        @type git_url: string
-
-        @param git_url: git url of repo (probably from github)
-
-        @type target_path: string
-
-        @param target_path: root path on remote server to copy git repo to
+        :type computer: string
+        :param computer: computer name to push to
+        :type user: string
+        :param user: user to own the files in the repo
+        :type group: string
+        :param group: group to own the files in the repo
+        :type git_url: string
+        :param git_url: git url of repo (probably from github)
+        :type target_path: string
+        :param target_path: root path on remote server to copy git repo to
         '''
         rsync_command = ('rsync -qrlptz --delete --delete-excluded '
                          '--exclude=.svn --exclude=.git')
@@ -642,13 +504,10 @@ class Recipe(object):
         '''
         Clone a git repo on a remote server.
 
-        @type git_url: string
-
-        @param git_url: git url of repo (probably from github)
-
-        @type target_path: string
-
-        @param target_path: root path on remote server to clone git repo into
+        :type git_url: string
+        :param git_url: git url of repo (probably from github)
+        :type target_path: string
+        :param target_path: root path on remote server to clone git repo into
         '''
         cuisine.sudo('sudo -Hi -u %s git clone %s %s' %
                      (user, git_url, target_path))
@@ -657,32 +516,25 @@ class Recipe(object):
         '''
         Update an existing git repo on a remote server.
 
-        @type git_url: string
-
-        @param git_url: git url of repo (probably from github)
-
-        @type target_path: string
-
-        @param target_path: root path on remote server to update git repo in
+        :type git_url: string
+        :param git_url: git url of repo (probably from github)
+        :type target_path: string
+        :param target_path: root path on remote server to update git repo in
         '''
         cuisine.sudo('cd %s && sudo -u %s git pull' % (target_path, user))
 
     def is_git_repo(self, target_path):
         '''
-        Make sure a git repo exists on the remote computer and is really
-        a git repo.
+        Make sure the target path exists on the remote computer and is
+        really a git repo.
 
-        @type git_url: string
+        :type git_url: string
+        :param git_url: git url of repo (probably from github)
+        :type target_path: string
+        :param target_path: root path on remote server to check git repo
 
-        @param git_url: git url of repo (probably from github)
-
-        @type target_path: string
-
-        @param target_path: root path on remote server to check git repo
-
-        @rtype: boolean
-
-        @return: True if repo already existed, False if not
+        :rtype: boolean
+        :return: True if repo already existed, False if not
 
         '''
         if not cuisine.dir_exists(os.path.join(target_path, '.git')):
@@ -694,19 +546,16 @@ class Recipe(object):
 
     def ensure_git_repo(self, user, git_url, target_path):
         '''
-        Make sure a git repo exists on the remote computer.
+        Make sure a git repo exists in the target path on the remote
+        computer.
 
-        @type git_url: string
+        :type git_url: string
+        :param git_url: git url of repo (probably from github)
+        :type target_path: string
+        :param target_path: root path on remote server to check git repo
 
-        @param git_url: git url of repo (probably from github)
-
-        @type target_path: string
-
-        @param target_path: root path on remote server to check git repo
-
-        @rtype: boolean
-
-        @return: True if repo already existed, False if not
+        :rtype: boolean
+        :return: True if repo already existed, False if not
         '''
         if not cuisine.dir_exists(target_path):
             self.clone_git_repo(user, git_url, target_path)
